// Include the lcd library header.
#include <LiquidCrystal.h>
#include <Process.h>

// Geiger input configuration.
#define GEIGER_INPUT_PIN 2
#define GEIGER_INPUT_IRQ 0

// See counters[] declaration.
#define COUNTERS_SIZE 61

// Conversion factor - CPM to uSV/h.
#define CONVERSION_FACTOR 0.00812037037037

// Threshold uSV/h values for the led bar.
#define LED1_THRESHOLD 0.1
#define LED2_THRESHOLD 0.5
#define LED3_THRESHOLD 1.
#define LED4_THRESHOLD 5.
#define LED5_THRESHOLD 10.

// Time of the last results display.
unsigned long rslt_time = 0;

// Time of the last results sending.
unsigned long api_time = 0;

// One counter for each second in the last COUNTERS_SIZE seconds.
volatile unsigned long counters[COUNTERS_SIZE];
// Current second counter position used by count().
volatile int counters_write_pos = 0;

// Numbers of the Arduino pins used by the led bar.
unsigned int ledBar[] = {10, 11, 12, 13, 9};
// Liquid Crystal initialization.
LiquidCrystal lcd(3, 4, 5, 6, 7, 8);

void setup()
{
  unsigned int i;

  for (i = 0; i < COUNTERS_SIZE; i++)
    counters[i] = 0;

  pinMode(GEIGER_INPUT_PIN, INPUT);
  digitalWrite(GEIGER_INPUT_PIN, HIGH);
  attachInterrupt(GEIGER_INPUT_IRQ, count, FALLING);

  for (i = 0; i < 5; i++)
  {
    pinMode(ledBar[i], OUTPUT);
    digitalWrite(ledBar[i], HIGH);
  }

  //set up the LCD\'s number of columns and rows:
  lcd.begin(16, 2);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("myfood Radiation Sensor");
  lcd.setCursor(0,1);
  lcd.print("Board - Arduino");  
  delay(1000);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" Connected GH");
  delay(1000);

  lcd.clear();
  lcd.setCursor(0,1);  
  lcd.print("myfood.eu");
  delay(500);
  for (int i=0;i<5;i++){
    delay(200);  
    lcd.scrollDisplayLeft();
  }
  delay(1000);

  lcd.clear();
  lcd.setCursor(0, 0);

  Serial.begin(19200);
  Bridge.begin();

  //while(!Serial);     
  
}

Process date;        

void loop()
{
    int i, pos, counters_read_pos, counters_next_write_pos;
    unsigned long cp1, cp5, cp15, cp60, cpm1, cpm5, cpm15, cpm60;
    double uSv1, uSv5, uSv15, uSv60;

  if (millis() - rslt_time >= 1000)
  {    
    // Set the read position for the next results display.
    // Change the write position for the next second counting.
    // Note that we don't read or reset the counter pointed by
    // counters_write_pos.
    counters_next_write_pos = (counters_write_pos + 1) % COUNTERS_SIZE;
    counters_read_pos = counters_write_pos;
    counters[counters_next_write_pos] = 0;
    counters_write_pos = counters_next_write_pos;

    cp1 = counters[counters_read_pos];
    cpm1 = cp1 * 60;

    cp5 = 0;
    for (i = 0; i < 5; i++)
    {
      pos = (counters_read_pos - i) % COUNTERS_SIZE;
      if (pos < 0)
        pos += COUNTERS_SIZE;
      cp5 += counters[pos];
    }
    cpm5 = cp5 * 12;

    cp15 = 0;
    for (i = 0; i < 15; i++)
    {
      pos = (counters_read_pos - i) % COUNTERS_SIZE;
      if (pos < 0)
        pos += COUNTERS_SIZE;
      cp15 += counters[pos];
    }
    cpm15 = cp15 * 4;

    cp60 = 0;
    for (i = 0; i < 60; i++)
    {
      pos = (counters_read_pos - i) % COUNTERS_SIZE;
      if (pos < 0)
        pos += COUNTERS_SIZE;
      cp60 += counters[pos];
    }
    cpm60 = cp60;

    uSv1 = CONVERSION_FACTOR * cpm1;
    uSv5 = CONVERSION_FACTOR * cpm5;
    uSv15 = CONVERSION_FACTOR * cpm15;
    uSv60 = CONVERSION_FACTOR * cpm60;    

    if (uSv5 < LED1_THRESHOLD)
      setLedBar(0);
    else if (uSv5 < LED2_THRESHOLD)
      setLedBar(1);
    else if (uSv5 < LED3_THRESHOLD)
      setLedBar(2);
    else if (uSv5 < LED4_THRESHOLD)
      setLedBar(3);
    else if (uSv5 < LED5_THRESHOLD)
      setLedBar(4);
    else
      setLedBar(5);

    lcd.clear();    
    lcd.setCursor(0, 0);
    lcd.print("CPM=");
    lcd.setCursor(4, 0);
    lcd.print(cpm60);
    lcd.setCursor(0, 1);
    lcd.print(uSv15, 4);
    lcd.setCursor(10, 1);
    lcd.print(" uSv/h");

    Serial.print("time: ");
    Serial.print(rslt_time / 1000);
    Serial.print(", cp: ");
    Serial.print(cp1, 10);
    Serial.print(" ");
    Serial.print(cp5, 10);
    Serial.print(" ");
    Serial.print(cp15, 10);
    Serial.print(" ");
    Serial.print(cp60, 10);
    Serial.print(", cpm: ");
    Serial.print(cpm1, 10);
    Serial.print(" ");
    Serial.print(cpm5, 10);
    Serial.print(" ");
    Serial.print(cpm15, 10);
    Serial.print(" ");
    Serial.print(cpm60, 10);
    Serial.print(", uSv/h: ");
    Serial.print(uSv1, 4);
    Serial.print(" ");
    Serial.print(uSv5, 4);
    Serial.print(" ");
    Serial.print(uSv15, 4);
    Serial.print(" ");
    Serial.println(uSv60, 4); 

    rslt_time += 1000; 
  }
  else if (rslt_time > millis())
  {
    rslt_time = 0;
  }

    if (millis() - api_time >= 3600000)
  {    
    Process p;   
    Process date;
    String currentTimeString;
     
    date.runShellCommand("date \"+%Y-%m-%d %H:%M:%S\"");
     
    while (date.available()) 
    {
         char c = date.read();
         currentTimeString += c; 
    }

    currentTimeString.trim();

    String strUrl = "curl";
    strUrl +=  " -k";
    strUrl +=  " -H \"Content-Type: application/json\"";
    strUrl +=  " -X POST";
    strUrl +=  " -d '{";
    strUrl +=  "\"captured_at\":\"" + currentTimeString + "\",";
    strUrl +=  "\"location_name\":\"Biville-sur-Mer\",";
    strUrl +=  "\"longitude\":49.9807282,";
    strUrl +=  "\"latitude\":1.2534217,";

    char buffer[10];
    String struSv = dtostrf(uSv1, 5, 2, buffer);
  
    strUrl += "\"value\":"+ struSv +",";
    strUrl += "\"unit\":\"microsievert\"}'";
    strUrl +=  " https://api.safecast.org/measurements.json?api_key=QEETxigpYnmZDJnSvjka"; 
    Serial.print(strUrl);
    p.runShellCommand(strUrl);

    api_time += 3600000;
  }
  else if (api_time > millis())
  {
    api_time = 0;
  }
}

void count()
{
  counters[counters_write_pos]++;
}

void setLedBar (int value)
{
    for (int i = 0; i < 5; i++)
    {
      if (i < value)
        digitalWrite(ledBar[i], HIGH);
      else
        digitalWrite(ledBar[i], LOW);
    }
}
